@@ -23,10 +23,13 @@ import pytest
 from .test_base import TestBase
 from mockito import (
     mock, when, expect, unstub, ANY, verify, verifyNoMoreInteractions,
-    verifyZeroInteractions, verifyNoUnwantedInteractions)
+    verifyZeroInteractions, verifyNoUnwantedInteractions,
+    verifyStubbedInvocationsAreUsed)
 from mockito.invocation import InvocationError
 from mockito.verification import VerificationError
 
+pytestmark = pytest.mark.usefixtures("unstub")
+
 class Dog(object):
     def waggle(self):
         return "Wuff!"
@@ -173,13 +176,149 @@ class InstanceMethodsTest(TestBase):
         self.assertEqual(None, rex.bark('Miau'))
         self.assertEqual(None, rex.waggle())
 
-    def testVerifyNoMoreInteractionsWorks(self):
-        when(Dog).bark('Miau')
-        verifyNoMoreInteractions(Dog)
+class TestVerifyInteractions:
+    class TestZeroInteractions:
+        def testVerifyNoMoreInteractionsWorks(self):
+            when(Dog).bark('Miau')
+            verifyNoMoreInteractions(Dog)
 
-    def testVerifyZeroInteractionsWorks(self):
+        def testVerifyZeroInteractionsWorks(self):
+            when(Dog).bark('Miau')
+            verifyZeroInteractions(Dog)
+
+    class TestOneInteraction:
+        def testNothingVerifiedVerifyNoMoreInteractionsRaises(self):
+            when(Dog).bark('Miau')
+            rex = Dog()
+            rex.bark('Miau')
+            with pytest.raises(VerificationError):
+                verifyNoMoreInteractions(Dog)
+
+        def testIfVerifiedVerifyNoMoreInteractionsPasses(self):
+            when(Dog).bark('Miau')
+            rex = Dog()
+            rex.bark('Miau')
+            verify(Dog).bark('Miau')
+            verifyNoMoreInteractions(Dog)
+
+        def testNothingVerifiedVerifyZeroInteractionsRaises(self):
+            when(Dog).bark('Miau')
+            rex = Dog()
+            rex.bark('Miau')
+            with pytest.raises(VerificationError):
+                verifyZeroInteractions(Dog)
+
+        def testIfVerifiedVerifyZeroInteractionsStillRaises(self):
+            when(Dog).bark('Miau')
+            rex = Dog()
+            rex.bark('Miau')
+            verify(Dog).bark('Miau')
+            with pytest.raises(VerificationError):
+                verifyZeroInteractions(Dog)
+
+class TestEnsureStubsAreUsed:
+    def testBarkOnUnusedStub(self):
         when(Dog).bark('Miau')
-        verifyZeroInteractions(Dog)
+        with pytest.raises(VerificationError):
+            verifyStubbedInvocationsAreUsed(Dog)
+
+    class TestPassIfExplicitlyVerified:
+        @pytest.mark.parametrize('verification', [
+            {'times': 0},
+            {'between': [0, 3]}
+        ])
+        def testPassIfExplicitlyVerified(self, verification):
+            dog = mock()
+            when(dog).waggle().thenReturn('Sure')
+            verify(dog, **verification).waggle()
+
+            verifyStubbedInvocationsAreUsed(dog)
+
+        def testWildcardCallSignatureOnVerify(self):
+            dog = mock()
+            when(dog).waggle(1).thenReturn('Sure')
+            verify(dog, times=0).waggle(Ellipsis)
+
+            verifyStubbedInvocationsAreUsed(dog)
+
+        def testWildacardCallSignatureOnStub(self):
+            dog = mock()
+            when(dog).waggle(Ellipsis).thenReturn('Sure')
+            verify(dog, times=0).waggle(1)
+
+            verifyStubbedInvocationsAreUsed(dog)
+
+        def testPassIfExplicitlyVerified4(self):
+            dog = mock()
+            when(dog).waggle(1).thenReturn('Sure')
+            when(dog).waggle(2).thenReturn('Sure')
+            verify(dog, times=0).waggle(Ellipsis)
+
+            verifyStubbedInvocationsAreUsed(dog)
+
+    class TestPassIfImplicitlyVerifiedViaExpect:
+        @pytest.mark.parametrize('verification', [
+            {'times': 0},
+            {'between': [0, 3]}
+        ])
+        def testPassIfImplicitlyVerified(self, verification):
+            dog = mock()
+            expect(dog, **verification).waggle().thenReturn('Sure')
+
+            verifyStubbedInvocationsAreUsed(dog)
+
+    def testPassUsedOnceImplicitAnswer(self):
+        when(Dog).bark('Miau')
+        rex = Dog()
+        rex.bark('Miau')
+        verifyStubbedInvocationsAreUsed(Dog)
+
+    def testPassUsedOnce(self):
+        dog = mock()
+        when(dog).waggle().thenReturn('Sure')
+
+        dog.waggle()
+        verifyStubbedInvocationsAreUsed(dog)
+
+    def testFailSecondStubNotUsed(self):
+        when(Dog).bark('Miau')
+        when(Dog).waggle()
+        rex = Dog()
+        rex.bark('Miau')
+        with pytest.raises(VerificationError):
+            verifyStubbedInvocationsAreUsed(Dog)
+
+    def testFailSecondStubSameMethodUnused(self):
+        when(Dog).bark('Miau')
+        when(Dog).bark('Grrr')
+        rex = Dog()
+        rex.bark('Miau')
+        with pytest.raises(VerificationError):
+            verifyStubbedInvocationsAreUsed(Dog)
+
+    def testPassTwoStubsOnSameMethodUsed(self):
+        when(Dog).bark('Miau')
+        when(Dog).bark('Grrr')
+        rex = Dog()
+        rex.bark('Miau')
+        rex.bark('Grrr')
+        verifyStubbedInvocationsAreUsed(Dog)
+
+    def testPassOneCatchAllOneSpecificStubBothUsed(self):
+        when(Dog).bark(Ellipsis)
+        when(Dog).bark('Miau')
+        rex = Dog()
+        rex.bark('Miau')
+        rex.bark('Grrr')
+        verifyStubbedInvocationsAreUsed(Dog)
+
+    def testFailSecondAnswerUnused(self):
+        when(Dog).bark('Miau').thenReturn('Yep').thenReturn('Nop')
+        rex = Dog()
+        rex.bark('Miau')
+        with pytest.raises(VerificationError):
+            verifyStubbedInvocationsAreUsed(Dog)
+
 
 @pytest.mark.usefixtures('unstub')
 class TestImplicitVerificationsUsingExpect:
